# Example script to show how to source the currently selected aperture from a PyJEM-equipped JEOL microscope. 
# Then select and insert the largest SAA aperture, and finally set the aperture selection back to what it originally was.
# This can be tested in an offline system. However, if you 
# test it at the microscope, ensure the .offline is removed (see below). You will then be able to look at the microscope interface
# while you run this and you will see the SAA aperture get seleted, the largest SA aperture inserted and
# the aperture currrently selected get restored to what it was previously. Remember to remove the SA aperture after testing.

# D. R. G. Mitchell, adminnospam@dmscripting.com (remove the nospam to make this address work)
# version:20220824, v1.0, August 2022, www.dmscripting.com

# Open this script in Spyder and monitor the output in the Console. If running at the microscope, monitor
# the microscope interface for changes in the aperture configuration.


# Import the TEM3 module from PyJEM. Here will be testing this offline, so use the PyJEM.offline module.
# To get this to run at the microscope remove the .offline part of the comman below.

from PyJEM.offline import TEM3
import time # we will use the sleep function in the time module to provide a delay
# when changing the apertures. This is to ensure it has enough time to respond. If you
# send too many commands to the microscope in quick succession, it may not have time to
# respond (aperture movement/selection can take several seconds). If it busy when a 
# command comes in, there will be a beep and an error message appears. Such errors are
# not fatal, but the command will be ignored and the action not completed. 


# Create an instance of the relevant aperture control module (Apt3)
# This provides access to all the aperture control commands.

my_apt3module=TEM3.Apt3()


# Get the currently selected aperture.
# The function returns an integer value specifying the aperture currently selected in the user interface
# 0=Nothing, 1=CLA, 2=OLA, 3=HCA, 4=SAA, 5=ENTA, 6=EDS
# The bright field STEM aperture does not appear to be accessible/controllable.

original_aperture_selected=my_apt3module.GetKind()
print("Selected aperture value = "+str(original_aperture_selected))


# store the aperture types in a list - in their numerical order
# The PyJEM literature refers to the EDS aperture, while in the microscope
# user interface this is labelled HXA - hard x-ray aperture)

list_of_apertures=['Nothing', 'CLA', 'OLA', 'HCA', 'SAA', 'ENTA', 'HXA']


# extract the relevant aperture type (name) from the list

aperture_type=list_of_apertures[original_aperture_selected]
print("Originally selected aperture is : "+aperture_type)


# To change a particular aperture you must first select it, then change it.
# eg to select the SAA aperture

my_apt3module.SelectKind(4) # 4 is the SAA aperture
print("Selecting the SAA aperture")
time.sleep(2) # provide a delay of 2s to allow the aperture to be selected. You may 
# hear a relay click, if the currently selected aperture was not the selected area aperture
# you will see the green selection LED on the SAA light up.


# Now insert the largest SAA aperture. Apertures are numbered 0 (Out), 1 (largest), 2 (2nd largest) etc. 

# Since the size, and potentially the number of apertures is user-customisable, a useful way of dealing with this is
# to customise your script so that you explicitly list which sized apertures you have on your microscope. This can be done in a list

saa_apertures=['Out','100','50','20','10'] # The size of the SA apertures on my F200


# This list can then be used to populate a popup menu in a dialog (See ApCon script).
# When the user selects a particular size from that dialog popup menu, the list above is interrogated to find
# its index (its position in the list, 0, 1, 2 etc) and that index is then used to 
# set the aperture size. It is important to ensure that the values used to set aperture positions remain within the
# range of available positions ie do not try and set the aperture position to 7
# if only four apertures positions are present. An error would result. Here we will just 
# specify a size of 1, which is the largest.

saa_position=1 # 1=largest aperture, 0 is out.
aperture_name=saa_apertures[saa_position] # get the name of the aperture at the specified position in the list
# Here we just use that name to print the size to the Console
print("Setting the SA aperture to "+str(aperture_name)+" um")
my_apt3module.SetSize(saa_position) # Ensure the position specified is available, otherwise an errror would occur.
time.sleep(2) # provide a delay or 2s to allow the aperture to respond.


# Finally, we will restore the aperture selection to what it was before running the script

my_apt3module.SelectKind(original_aperture_selected)
print("Restoring the aperture selection to "+str(aperture_type))
time.sleep(2)
# if your script ends here, you do not need the sleep. However, if your code were to continue
# and interact with other parts of the microscope, then the delay may be necessary. 
print("Remember to manually remove the SA Aperture when finished.")